"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnphaseDataService = void 0;
const axios_1 = __importDefault(require("axios"));
const axios_digest_auth_1 = __importDefault(require("@mhoc/axios-digest-auth"));
const fs = require("fs");
const https = require("https");
const tesla_service_1 = require("../tesla/tesla.service");
const configuration_service_1 = require("../services/configuration.service");
const myEnergi_service_1 = require("../myEnergi/myEnergi.service");
class EnphaseDataService {
    constructor() {
    }
    static init() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Init of EnphaseDataService');
            // Read previous enphase datas
            try {
                let str = fs.readFileSync('./data/enphase_data.json', 'utf8');
                EnphaseDataService.enphaseData = JSON.parse(str);
                if (!EnphaseDataService.enphaseData.data_detail) {
                    EnphaseDataService.enphaseData.data_detail = [];
                }
                if (EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_exported) {
                    EnphaseDataService.lastExportationWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_exported;
                }
                else {
                    for (let i = 0; i < EnphaseDataService.enphaseData.data_detail.length; i++) {
                        EnphaseDataService.enphaseData.data_detail[i].wh_exported = 0;
                    }
                    for (let i = 0; i < EnphaseDataService.enphaseData.data.length; i++) {
                        EnphaseDataService.enphaseData.data[i].wh_exported = 0;
                    }
                }
                EnphaseDataService.lastProductionWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_producted;
                EnphaseDataService.lastConsomptionWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_consumed;
                EnphaseDataService.lastExportationWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_exported;
                console.log('Previous data loaded');
            }
            catch (e) {
                EnphaseDataService.enphaseData = { data: [], data_detail: [] };
                console.log('Unable to load the previous data');
                EnphaseDataService.needData = true;
            }
            // Get a cookie for this session
            yield EnphaseDataService.getCookie();
            // Try to get a new cookie every 4 hours
            //setInterval( ()=>{EnphaseDataService.getCookie();}, 1000*30 );
            setInterval(() => { EnphaseDataService.getCookie(); }, 1000 * 60 * 60 * 4);
            // Launch the get data polling
            EnphaseDataService.poll();
        });
    }
    static setConfiguration(configuration, attribModified, from) {
        if (attribModified.startsWith('"enphase.') && from !== 'EnphaseDataService') {
            EnphaseDataService.getCookie();
        }
    }
    static getCookie() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (configuration_service_1.ConfigurationService.getConfiguration().enphase.version === '5') {
                return;
            }
            // Try to get new token
            let token = yield EnphaseDataService.getNewToken();
            if (token.length > 0) {
                configuration_service_1.ConfigurationService.getConfiguration().enphase.token = token;
                configuration_service_1.ConfigurationService.setConfiguration(configuration_service_1.ConfigurationService.getConfiguration(), 'enphase.token', 'EnphaseDataService');
                this.parseJwt(configuration_service_1.ConfigurationService.getConfiguration().enphase.token);
            }
            const agent = new https.Agent({
                rejectUnauthorized: false
            });
            try {
                let url = 'https://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/auth/check_jwt';
                let response = yield axios_1.default.get(url, { 'headers': { 'Authorization': 'Bearer ' + configuration_service_1.ConfigurationService.getConfiguration().enphase.token, 'Content-Type': 'application/json' }, httpsAgent: agent }).catch(error => {
                    console.error('Error axios to get enphase token (url: ' + url + ') : ' + error.toString());
                });
                if ((response === null || response === void 0 ? void 0 : response.status) === 200) {
                    EnphaseDataService.cookie = (_a = response.headers["set-cookie"]) === null || _a === void 0 ? void 0 : _a.at(0);
                    console.log('Get new cookie from token succeeded');
                }
            }
            catch (e) {
                console.error('Unable to get the cookie from token : ' + e);
            }
        });
    }
    // use to know information about token
    static parseJwt(str) {
        let tokenInfo = JSON.parse(Buffer.from(str.split('.')[1], 'base64').toString());
        console.log('token expiration date : ' + (new Date(tokenInfo.exp * 1000)).toLocaleString());
    }
    static getNewToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = new https.Agent({
                rejectUnauthorized: false
            });
            if (!configuration_service_1.ConfigurationService.getConfiguration().enphase.userEnphaseEnergy || configuration_service_1.ConfigurationService.getConfiguration().enphase.userEnphaseEnergy === '') {
                console.log('Pas d\'obtention du token car le champs utilisateur n\'est pas rempli');
                return '';
            }
            if (!configuration_service_1.ConfigurationService.getConfiguration().enphase.passwordEnphaseEnergy || configuration_service_1.ConfigurationService.getConfiguration().enphase.passwordEnphaseEnergy === '') {
                console.log('Pas d\'obtention du token car le champs mot de passe n\'est pas rempli');
                return '';
            }
            if (!configuration_service_1.ConfigurationService.getConfiguration().enphase.gatewaySerial || configuration_service_1.ConfigurationService.getConfiguration().enphase.gatewaySerial === '') {
                console.log('Pas d\'obtention du token car le champs numéro de série n\'est pas rempli');
                return '';
            }
            // Try to get sesion id
            let response = yield axios_1.default.post('https://enlighten.enphaseenergy.com/login/login.json', 'user[email]=' + configuration_service_1.ConfigurationService.getConfiguration().enphase.userEnphaseEnergy + '&user[password]=' + configuration_service_1.ConfigurationService.getConfiguration().enphase.passwordEnphaseEnergy, { httpsAgent: agent }).catch(error => {
                console.error('Error axios to get enphase session id');
            });
            if ((response === null || response === void 0 ? void 0 : response.status) === 200) {
                console.log('Get session id from enphaseenergy success');
                // Get token
                response = yield axios_1.default.get('https://enlighten.enphaseenergy.com/entrez-auth-token?serial_num=' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gatewaySerial, { 'headers': { 'Cookie': '_enlighten_4_session=' + response.data.session_id, 'Content-Type': 'application/json' }, httpsAgent: agent }).catch(error => {
                    console.error('Error axios to get enphase token');
                });
                if ((response === null || response === void 0 ? void 0 : response.status) === 200) {
                    console.log('Get token from session id success');
                    return response.data.token;
                }
                else {
                    console.error('Get token from session id failed');
                }
            }
            else {
                console.error('Get session id from enphaseenergy failed');
            }
            return '';
        });
    }
    static poll() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let lastGetDataTime = new Date().getTime();
            let response = yield EnphaseDataService.getData();
            if (!response) {
                // Try to get data every 10 seconds
                setTimeout(EnphaseDataService.poll, 10 * 1000);
                return;
            }
            let zappiData = null;
            if ((_a = configuration_service_1.ConfigurationService.getConfiguration().myEnergi) === null || _a === void 0 ? void 0 : _a.useMyEnergi) {
                zappiData = yield myEnergi_service_1.MyEnergiService.getNewData();
            }
            if (response.status === 200 && response.data.production) {
                console.log('production : ' + response.data.production[1].wNow + ' (lifetime:' + response.data.production[1].whLifetime + ')');
                let production = response.data.production.filter((production) => production.type === 'eim')[0];
                let consumption = response.data.consumption.filter((consumption) => consumption.type === 'eim' && consumption.measurementType === 'total-consumption')[0];
                let productionWhLifetime = production.whLifetime;
                let readingTime = production.readingTime;
                EnphaseDataService.lastReadingTime = production.readingTime;
                let consumptionWhLifetime = consumption.whLifetime;
                let w_charge = 0;
                let w_route = 0;
                if (zappiData !== null) {
                    w_charge = zappiData.ectp1;
                    w_route = zappiData.ectp4;
                }
                if (EnphaseDataService.needData) {
                    EnphaseDataService.needData = false;
                    EnphaseDataService.enphaseData.data.push({ timestamp: readingTime, wh_producted: productionWhLifetime, wh_consumed: consumptionWhLifetime, wh_exported: EnphaseDataService.lastExportationWhLifetime });
                    // Save datas
                    EnphaseDataService.save();
                }
                else {
                    // Compute exportation if any
                    if ((productionWhLifetime - EnphaseDataService.lastProductionWhLifetime) > (consumptionWhLifetime - EnphaseDataService.lastConsomptionWhLifetime)) {
                        EnphaseDataService.lastExportationWhLifetime += (productionWhLifetime - EnphaseDataService.lastProductionWhLifetime) - (consumptionWhLifetime - EnphaseDataService.lastConsomptionWhLifetime);
                    }
                    // Add data to detail array every 1 minute
                    if (EnphaseDataService.enphaseData.data_detail.length === 0 || (readingTime - EnphaseDataService.enphaseData.data_detail[EnphaseDataService.enphaseData.data_detail.length - 1].timestamp) >= 60) {
                        EnphaseDataService.enphaseData.data_detail.push({ timestamp: readingTime, wh_producted: productionWhLifetime, wh_consumed: consumptionWhLifetime, wh_exported: EnphaseDataService.lastExportationWhLifetime });
                    }
                    // Add data to extradetail array every time
                    EnphaseDataService.extraDetail.push({ timestamp: readingTime, wh_producted: productionWhLifetime, wh_consumed: consumptionWhLifetime, wh_exported: EnphaseDataService.lastExportationWhLifetime });
                    if (EnphaseDataService.extraDetail.length > 600) {
                        EnphaseDataService.extraDetail.splice(0, EnphaseDataService.extraDetail.length - 600);
                    }
                    // Remove detail if necessary
                    while (EnphaseDataService.enphaseData.data_detail.length > 0 && ((new Date().getTime() / 1000) - EnphaseDataService.enphaseData.data_detail[0].timestamp) > 48 * 60 * 60) {
                        EnphaseDataService.enphaseData.data_detail.splice(0, 1);
                    }
                    // Add data to global array every 10 minutes
                    if ((readingTime - EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].timestamp) > 10 * 60) {
                        EnphaseDataService.enphaseData.data.push({ timestamp: readingTime, wh_producted: productionWhLifetime, wh_consumed: consumptionWhLifetime, wh_exported: EnphaseDataService.lastExportationWhLifetime });
                        // Save datas
                        EnphaseDataService.save();
                    }
                }
                // Get data every second
                let delay = Math.floor(1000 - (new Date().getTime() - lastGetDataTime));
                if (delay < 1) {
                    delay = 1;
                }
                // Save real time data
                EnphaseDataService.currentProduction = production.wNow;
                EnphaseDataService.currentConsumption = consumption.wNow;
                EnphaseDataService.lastProductionWhLifetime = productionWhLifetime;
                EnphaseDataService.lastConsomptionWhLifetime = consumptionWhLifetime;
                if (production && consumption) {
                    EnphaseDataService.realTimeArray.push({ production: { timestamp: production.readingTime, wNow: production.wNow }, consumption: { timestamp: consumption.readingTime, wNow: consumption.wNow } });
                    // Remove all data more than 1 hour
                    while (EnphaseDataService.realTimeArray.length > 0 && EnphaseDataService.realTimeArray[0].production.timestamp < production.readingTime - 3600) {
                        EnphaseDataService.realTimeArray.splice(0, 1);
                    }
                }
                console.log('enphasee real time size : ' + EnphaseDataService.realTimeArray.length);
                if (configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
                    tesla_service_1.TeslaService.checkModeSolarPlus();
                }
                if ((_b = configuration_service_1.ConfigurationService.getConfiguration().myEnergi) === null || _b === void 0 ? void 0 : _b.useMyEnergi) {
                    myEnergi_service_1.MyEnergiService.checkModeSolarPlus();
                }
                setTimeout(EnphaseDataService.poll, delay);
            }
            else {
                // retry to get data
                setTimeout(EnphaseDataService.poll, 5 * 1000);
            }
        });
    }
    static getProductionRealTime() {
        return {
            w_production: EnphaseDataService.currentProduction,
            w_consumption: EnphaseDataService.currentConsumption
        };
    }
    static getProductionRealTimeOneHour() {
        return EnphaseDataService.realTimeArray;
    }
    static getData() {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = new https.Agent({
                rejectUnauthorized: false
            });
            let response;
            try {
                if (configuration_service_1.ConfigurationService.getConfiguration().enphase.version === '5') {
                    response = yield axios_1.default.get('http://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/production.json', { 'headers': { 'Content-Type': 'application/json' } }).catch(error => {
                        console.error('Error axios to get enphase data');
                    });
                }
                else {
                    response = yield axios_1.default.get('https://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/production.json', { 'headers': { 'Cookie': EnphaseDataService.cookie, 'Content-Type': 'application/json' }, httpsAgent: agent }).catch(error => {
                        console.error('Error axios to get enphase data');
                    });
                }
            }
            catch (error) {
                console.error('Error to get data form enphase gateway');
                try {
                    if (configuration_service_1.ConfigurationService.getConfiguration().enphase.version !== '5') {
                        yield EnphaseDataService.getCookie();
                        response = yield axios_1.default.get('https://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/production.json', { 'headers': { 'Cookie': EnphaseDataService.cookie, 'Content-Type': 'application/json' }, httpsAgent: agent }).catch(error => {
                            console.error('Error axios to get enphase cookie');
                        });
                    }
                }
                catch (error2) {
                    console.error('Error to get cookie and data from enphase gateway');
                }
            }
            if (response) {
                return response;
            }
            return null;
        });
    }
    static save() {
        console.log('Save datas');
        fs.writeFileSync('./data/enphase_data.json', JSON.stringify(EnphaseDataService.enphaseData, null, 2));
    }
    static saveBackup() {
        console.log('Save datas as backup');
        let now = new Date();
        let filename = './data/enphase_data_';
        filename += now.getFullYear();
        filename += '-' + EnphaseDataService.pad(now.getMonth() + 1, 2);
        filename += '-' + EnphaseDataService.pad(now.getDate(), 2);
        filename += '_' + EnphaseDataService.pad(now.getHours(), 2);
        filename += '-' + EnphaseDataService.pad(now.getMinutes(), 2);
        filename += '-' + EnphaseDataService.pad(now.getSeconds(), 2);
        filename += '.json';
        fs.writeFileSync(filename, JSON.stringify(EnphaseDataService.enphaseData, null, 2));
    }
    /**
     * Add zero at begining of number if necessary to get a string of length = size
     *
     * @param num
     * @param size
     * @returns
     */
    static pad(num, size) {
        let numStr = num.toString();
        while (numStr.length < size)
            numStr = '0' + numStr;
        return numStr;
    }
    static getStepsValues(steps) {
        let data = EnphaseDataService.enphaseData.data;
        // Get first occurence before begin
        let currentIndex = EnphaseDataService.getIndexBefore(data, steps[0].begin);
        let result = [];
        let previousEnphaseValue = EnphaseDataService.getValueAt(data, steps[0].begin, currentIndex);
        for (let i = 0; i < steps.length; i++) {
            let value = EnphaseDataService.getValueAt(data, steps[i].end, currentIndex);
            let enphaseStep = {
                timestamp: steps[i].end,
                w_consumed: (value.wh_consumed - previousEnphaseValue.wh_consumed) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                w_producted: (value.wh_producted - previousEnphaseValue.wh_producted) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                w_exported: (value.wh_exported - previousEnphaseValue.wh_exported) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                wh_consumed: value.wh_consumed - previousEnphaseValue.wh_consumed,
                wh_producted: value.wh_producted - previousEnphaseValue.wh_producted,
                wh_exported: value.wh_exported - previousEnphaseValue.wh_exported,
                wh_producted_lifetime: value.wh_producted,
                wh_consumed_lifetime: value.wh_consumed,
                wh_exported_lifetime: value.wh_exported
            };
            if (value.wh_consumed === -1) {
                // Special case when no data available
                enphaseStep.w_consumed = 0;
                enphaseStep.w_exported = 0;
                enphaseStep.w_producted = 0;
                enphaseStep.wh_consumed = 0;
                enphaseStep.wh_exported = 0;
                enphaseStep.wh_producted = 0;
                enphaseStep.wh_consumed_lifetime = previousEnphaseValue.wh_consumed;
                enphaseStep.wh_producted_lifetime = previousEnphaseValue.wh_producted;
                enphaseStep.wh_exported_lifetime = previousEnphaseValue.wh_exported;
                value.wh_consumed = previousEnphaseValue.wh_consumed;
                value.wh_producted = previousEnphaseValue.wh_producted;
            }
            result.push(enphaseStep);
            previousEnphaseValue = value;
        }
        return result;
    }
    static getBetweenDate(begin, end) {
        let data = EnphaseDataService.enphaseData.data;
        data = data.concat(EnphaseDataService.enphaseData.data_detail);
        data = data.concat(EnphaseDataService.extraDetail);
        // Sort array
        data.sort((data1, data2) => {
            if (data1.timestamp > data2.timestamp) {
                return 1;
            }
            else if (data1.timestamp < data2.timestamp) {
                return -1;
            }
            return 0;
        });
        let valueBegin = EnphaseDataService.getValueAt(data, begin, 0);
        let valueEnd = EnphaseDataService.getValueAt(data, end, 0);
        let enphaseStep = {
            timestamp: end,
            w_consumed: (valueEnd.wh_consumed - valueBegin.wh_consumed) / (valueEnd.timestamp - valueBegin.timestamp) * 3600,
            w_producted: (valueEnd.wh_producted - valueBegin.wh_producted) / (valueEnd.timestamp - valueBegin.timestamp) * 3600,
            w_exported: (valueEnd.wh_exported - valueBegin.wh_exported) / (valueEnd.timestamp - valueBegin.timestamp) * 3600,
            wh_consumed: valueEnd.wh_consumed - valueBegin.wh_consumed,
            wh_producted: valueEnd.wh_producted - valueBegin.wh_producted,
            wh_exported: valueEnd.wh_exported - valueBegin.wh_exported,
            wh_producted_lifetime: valueEnd.wh_producted,
            wh_consumed_lifetime: valueEnd.wh_consumed,
            wh_exported_lifetime: valueEnd.wh_exported
        };
        return enphaseStep;
    }
    static getSteps(begin, end, step) {
        let data = EnphaseDataService.enphaseData.data;
        // If data is less than 48h then get details data
        if ((begin - step) > ((new Date().getTime() / 1000) - 48 * 60 * 60)) {
            // In this case build a data with all detail data and global data
            data = [];
            let index = EnphaseDataService.getIndexBefore(EnphaseDataService.enphaseData.data, begin - step);
            while (index < EnphaseDataService.enphaseData.data.length) {
                data.push(EnphaseDataService.enphaseData.data[index]);
                index++;
            }
            // Add detail and extra detail data
            data = data.concat(EnphaseDataService.enphaseData.data_detail);
            data = data.concat(EnphaseDataService.extraDetail);
            // Sort array
            data.sort((data1, data2) => {
                if (data1.timestamp > data2.timestamp) {
                    return 1;
                }
                else if (data1.timestamp < data2.timestamp) {
                    return -1;
                }
                return 0;
            });
        }
        // Get first occurence before begin
        let currentIndex = EnphaseDataService.getIndexBefore(data, begin - step);
        let result = [];
        let previousEnphaseValue = EnphaseDataService.getValueAt(data, begin - step, currentIndex);
        let currentTime = begin;
        while (currentTime <= end) {
            let value = EnphaseDataService.getValueAt(data, currentTime, currentIndex);
            let enphaseStep = {
                timestamp: currentTime,
                w_consumed: (value.wh_consumed - previousEnphaseValue.wh_consumed) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                w_producted: (value.wh_producted - previousEnphaseValue.wh_producted) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                w_exported: (value.wh_exported - previousEnphaseValue.wh_exported) / (value.timestamp - previousEnphaseValue.timestamp) * 3600,
                wh_consumed: value.wh_consumed - previousEnphaseValue.wh_consumed,
                wh_producted: value.wh_producted - previousEnphaseValue.wh_producted,
                wh_exported: value.wh_exported - previousEnphaseValue.wh_exported,
                wh_producted_lifetime: value.wh_producted,
                wh_consumed_lifetime: value.wh_consumed,
                wh_exported_lifetime: value.wh_exported
            };
            if (value.wh_consumed === -1) {
                // Special case when no data available
                enphaseStep.w_consumed = 0;
                enphaseStep.w_exported = 0;
                enphaseStep.w_producted = 0;
                enphaseStep.wh_consumed = 0;
                enphaseStep.wh_exported = 0;
                enphaseStep.wh_producted = 0;
                enphaseStep.wh_consumed_lifetime = previousEnphaseValue.wh_consumed;
                enphaseStep.wh_producted_lifetime = previousEnphaseValue.wh_producted;
                enphaseStep.wh_exported_lifetime = previousEnphaseValue.wh_exported;
                value.wh_consumed = previousEnphaseValue.wh_consumed;
                value.wh_producted = previousEnphaseValue.wh_producted;
            }
            result.push(enphaseStep);
            currentTime += step;
            previousEnphaseValue = value;
        }
        return result;
    }
    static getValueAt(data, timestamp, index) {
        let result;
        if (data.length === 0) {
            result = {
                timestamp: timestamp,
                wh_consumed: 0,
                wh_producted: 0,
                wh_exported: 0
            };
            return result;
        }
        let firstIndex = index;
        while (firstIndex + 1 < data.length && data[firstIndex + 1].timestamp < timestamp) {
            firstIndex++;
        }
        if (firstIndex + 1 >= data.length) {
            return {
                timestamp: timestamp,
                wh_consumed: data[data.length - 1].wh_consumed,
                wh_producted: data[data.length - 1].wh_producted,
                wh_exported: data[data.length - 1].wh_exported,
            };
        }
        let coef = (timestamp - data[firstIndex].timestamp) / (data[firstIndex + 1].timestamp - data[firstIndex].timestamp);
        result = {
            timestamp: timestamp,
            wh_consumed: data[firstIndex].wh_consumed + (data[firstIndex + 1].wh_consumed - data[firstIndex].wh_consumed) * coef,
            wh_producted: data[firstIndex].wh_producted + (data[firstIndex + 1].wh_producted - data[firstIndex].wh_producted) * coef,
            wh_exported: data[firstIndex].wh_exported + (data[firstIndex + 1].wh_exported - data[firstIndex].wh_exported) * coef
        };
        if (timestamp < data[firstIndex].timestamp) {
            result = {
                timestamp: timestamp,
                wh_consumed: data[firstIndex].wh_consumed,
                wh_producted: data[firstIndex].wh_producted,
                wh_exported: data[firstIndex].wh_exported
            };
        }
        return result;
    }
    static getIndexBefore(data, timestamp) {
        return EnphaseDataService.getIndexBeforeStep2(data, timestamp, 0, data.length - 1);
    }
    static getIndexBeforeStep2(data, timestamp, firstIndex, lastIndex) {
        if (firstIndex === lastIndex) {
            return firstIndex;
        }
        else if (lastIndex - firstIndex === 1) {
            if (data[lastIndex].timestamp < timestamp) {
                return lastIndex;
            }
            else {
                return firstIndex;
            }
        }
        let middleIndex = Math.floor((firstIndex + lastIndex) / 2);
        if (data[middleIndex] === undefined) {
            return firstIndex;
        }
        if (data[middleIndex].timestamp > timestamp) {
            return EnphaseDataService.getIndexBeforeStep2(data, timestamp, firstIndex, middleIndex);
        }
        return EnphaseDataService.getIndexBeforeStep2(data, timestamp, middleIndex, lastIndex);
    }
    static getProductionInverter() {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = new https.Agent({
                rejectUnauthorized: false
            });
            let response = null;
            try {
                if (configuration_service_1.ConfigurationService.getConfiguration().enphase.version === '5') {
                    const digestAuth = new axios_digest_auth_1.default({
                        username: configuration_service_1.ConfigurationService.getConfiguration().enphase.user,
                        password: configuration_service_1.ConfigurationService.getConfiguration().enphase.password,
                    });
                    response = digestAuth.request({ headers: { Accept: 'application/json' }, method: "GET", url: 'http://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/api/v1/production/inverters', });
                }
                else {
                    response = yield axios_1.default.get('https://' + configuration_service_1.ConfigurationService.getConfiguration().enphase.gateway + '/api/v1/production/inverters', { 'headers': { 'Cookie': EnphaseDataService.cookie, 'Content-Type': 'application/json' }, httpsAgent: agent });
                }
                // To get data from stream
                /*let response2 = await axios.get(EnphaseDataService.configuration.enphase.gateway + 'stream/meter',{ headers: { Authorization: '' ,Cookie: EnphaseDataService.cookie, 'Content-Type': 'application/json'}, responseType: 'stream' , httpsAgent: agent });
                let stream = response2.data;
                stream.on('data', (data:Uint8Array) => {
                    console.log( "receive : " + data.toString() );
                });*/
            }
            catch (error) {
                // We will get better chance next time
                console.error("error to read api/v1/production/inverters");
            }
            return response;
        });
    }
    static getOverproduction(fromTime) {
        if (fromTime === 0) {
            return EnphaseDataService.currentProduction - EnphaseDataService.currentConsumption;
        }
        if (fromTime < 10) {
            fromTime = 10;
        }
        if (EnphaseDataService.extraDetail.length < 2) {
            return null;
        }
        let detail2 = EnphaseDataService.extraDetail[EnphaseDataService.extraDetail.length - 1];
        let detail1 = null;
        for (let i = EnphaseDataService.extraDetail.length - 2; i >= 0; i--) {
            if (EnphaseDataService.extraDetail[i].timestamp > (detail2.timestamp - fromTime)) {
                detail1 = EnphaseDataService.extraDetail[i];
            }
            else {
                break;
            }
        }
        if (detail1) {
            let overProduction = ((detail2.wh_producted - detail1.wh_producted) - (detail2.wh_consumed - detail1.wh_consumed)) / (detail2.timestamp - detail1.timestamp) * 3600;
            return overProduction;
        }
        return null;
    }
    static getStartTimestamp() {
        if (EnphaseDataService.enphaseData.data.length > 0) {
            return EnphaseDataService.enphaseData.data[0].timestamp;
        }
        return 0;
    }
    static getLastReadingTime() {
        return EnphaseDataService.lastReadingTime;
    }
    static importFile(importStr, confirm) {
        let importData = [];
        let strResult = '';
        // First set import data
        if (importStr) {
            let producted_lifetime = 0;
            let consumed_lifetime = 0;
            let exported_lifetime = 0;
            let strArray1 = importStr.split('\r\n');
            let strArray2 = importStr.split('\n');
            let strArray = strArray1;
            if (strArray2.length > strArray.length) {
                strArray = strArray2;
            }
            for (let i = 1; i < strArray.length - 1; i++) {
                let dataArray = strArray[i].split(',');
                let timestamp = Math.floor(new Date(dataArray[0]).getTime() / 1000);
                let productValue = parseInt(dataArray[1]);
                let consumedValue = parseInt(dataArray[2]);
                let exportedValue = parseInt(dataArray[3]);
                producted_lifetime += productValue;
                consumed_lifetime += consumedValue;
                exported_lifetime += exportedValue;
                importData.push({ timestamp: timestamp, wh_producted: producted_lifetime, wh_consumed: consumed_lifetime, wh_exported: exported_lifetime, wh_period_exported: exportedValue });
            }
        }
        // Remove all 0 value at beginning
        while (importData[1].wh_producted === 0 && importData[1].wh_consumed === 0 && importData[1].wh_exported === 0) {
            importData.splice(0, 1);
        }
        // Check difference for last data of import data
        let dif_producted = importData[importData.length - 1].wh_producted - EnphaseDataService.getProductedAtTimestamp(importData[importData.length - 1].timestamp);
        let dif_exported = importData[importData.length - 1].wh_exported - EnphaseDataService.getExportedAtTimestamp(importData[importData.length - 1].timestamp);
        let dif_consumed = importData[importData.length - 1].wh_consumed - EnphaseDataService.getConsumedAtTimestamp(importData[importData.length - 1].timestamp);
        strResult += 'Difference of producted : ' + Math.floor(dif_producted) + ' Wh<br>';
        strResult += 'Difference of consumed : ' + Math.floor(dif_consumed) + ' Wh<br>';
        strResult += 'Difference of exported (big number is normal): ' + Math.floor(dif_exported) + ' Wh<br>';
        // Compute empty period
        let data = EnphaseDataService.enphaseData.data;
        if (data[0].timestamp - importData[0].timestamp > 10 * 60) {
            let str = EnphaseDataService.computeStrDate(importData[0].timestamp, data[0].timestamp) + '<br>';
            if (!confirm) {
                strResult += str;
            }
        }
        for (let i = 0; i < data.length - 1; i++) {
            if (data[i + 1].timestamp > data[i].timestamp + 20 * 60) {
                let str = EnphaseDataService.computeStrDate(data[i].timestamp, data[i + 1].timestamp) + '<br>';
                if (!confirm) {
                    strResult += str;
                }
            }
        }
        if (confirm) {
            // Save current data
            EnphaseDataService.saveBackup();
            // Check that data are from beginning
            let difProd = importData[importData.length - 1].wh_producted - EnphaseDataService.getProductedAtTimestamp(importData[importData.length - 1].timestamp);
            let difCons = importData[importData.length - 1].wh_consumed - EnphaseDataService.getConsumedAtTimestamp(importData[importData.length - 1].timestamp);
            let difExp = importData[importData.length - 1].wh_exported - EnphaseDataService.getExportedAtTimestamp(importData[importData.length - 1].timestamp);
            console.log('tttt ' + difProd + '    ' + difCons + '    ' + difExp);
            for (let i = 0; i < importData.length; i++) {
                importData[i].wh_producted -= difProd;
                importData[i].wh_consumed -= difCons;
                importData[i].wh_exported -= difExp;
                //console.log( '2 ' + importData[i].wh_producted + '    ' + importData[i].wh_exported + '    ' + importData[i].wh_consumed );
            }
            // Now add import data into data
            if (data[0].timestamp - importData[0].timestamp > 10 * 60) {
                strResult += 'Import data<BR>';
                strResult += EnphaseDataService.computeNewData(importData[0].timestamp, data[0].timestamp, importData) + '<br>';
            }
            let i = 1;
            while (i < data.length - 1) {
                if (data[i + 1].timestamp > data[i].timestamp + 20 * 60) {
                    strResult += EnphaseDataService.computeNewData(data[i].timestamp, data[i + 1].timestamp, importData) + '<br>';
                }
                i++;
            }
            EnphaseDataService.save();
            dif_producted = importData[importData.length - 1].wh_producted - EnphaseDataService.getProductedAtTimestamp(importData[importData.length - 1].timestamp);
            dif_exported = importData[importData.length - 1].wh_exported - EnphaseDataService.getExportedAtTimestamp(importData[importData.length - 1].timestamp);
            dif_consumed = importData[importData.length - 1].wh_consumed - EnphaseDataService.getConsumedAtTimestamp(importData[importData.length - 1].timestamp);
            if (confirm) {
                strResult += 'Difference of producted after merge : ' + Math.floor(dif_producted) + ' Wh<br>';
                strResult += 'Difference of consumed after merge : ' + Math.floor(dif_consumed) + ' Wh<br>';
                strResult += 'Difference of exported after merge : ' + Math.floor(dif_exported) + ' Wh<br>';
            }
        }
        return strResult;
    }
    static computeNewData(timestamp1, timestamp2, importData) {
        if (importData[0].timestamp >= timestamp2) {
            let str = this.computeImportStrDate(timestamp1, timestamp2, 0);
            return str;
        }
        if (timestamp1 > EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].timestamp) {
            let str = this.computeImportStrDate(timestamp1, timestamp2, 0);
            return str;
        }
        if (timestamp2 > EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].timestamp) {
            let str = this.computeImportStrDate(timestamp1, timestamp2, 0);
            return str;
        }
        // First, find begin import data
        let firstImportDataIndex = -1;
        for (let i = 0; i < importData.length; i++) {
            if (importData[i].timestamp > timestamp1) {
                firstImportDataIndex = i;
                break;
            }
        }
        if (firstImportDataIndex === -1) {
            return 'Unable to import data because start not in import file for period : ' + new Date(timestamp1 * 1000).toLocaleDateString() + ' ' + new Date(timestamp1 * 1000).toLocaleTimeString() + ' - ' + new Date(timestamp2 * 1000).toLocaleDateString() + ' ' + new Date(timestamp2 * 1000).toLocaleTimeString();
        }
        // Then find end import data
        let lastImportDataIndex = -1;
        for (let i = firstImportDataIndex + 1; i < importData.length; i++) {
            if (importData[i].timestamp > timestamp2) {
                lastImportDataIndex = i - 1;
                break;
            }
        }
        if (lastImportDataIndex === -1) {
            return 'Unable to import data because end not in import file for period : ' + new Date(timestamp1 * 1000).toLocaleDateString() + ' ' + new Date(timestamp1 * 1000).toLocaleTimeString() + ' - ' + new Date(timestamp2 * 1000).toLocaleDateString() + ' ' + new Date(timestamp2 * 1000).toLocaleTimeString();
        }
        // Then find where to insert data
        let insertAtIndex = -1;
        for (let i = 0; i < EnphaseDataService.enphaseData.data.length; i++) {
            if (EnphaseDataService.enphaseData.data[i].timestamp > timestamp1) {
                insertAtIndex = i;
                break;
            }
        }
        if (insertAtIndex === -1) {
            return 'Unable to import data because index not found in current data for period : ' + new Date(timestamp1 * 1000).toLocaleDateString() + ' ' + new Date(timestamp1 * 1000).toLocaleTimeString() + ' - ' + new Date(timestamp2 * 1000).toLocaleDateString() + ' ' + new Date(timestamp2 * 1000).toLocaleTimeString();
        }
        // Now, import data
        let min_wh_producted = 0;
        let min_wh_consumed = 0;
        let min_wh_exported = 0;
        let current_wh_exported = 0;
        let total_exported = 0;
        if (insertAtIndex > 0) {
            current_wh_exported = EnphaseDataService.enphaseData.data[insertAtIndex - 1].wh_exported;
            min_wh_producted = EnphaseDataService.enphaseData.data[insertAtIndex - 1].wh_producted;
            min_wh_consumed = EnphaseDataService.enphaseData.data[insertAtIndex - 1].wh_consumed;
            min_wh_exported = EnphaseDataService.enphaseData.data[insertAtIndex - 1].wh_exported;
        }
        let max_wh_producted = EnphaseDataService.enphaseData.data[insertAtIndex].wh_producted;
        let max_wh_consumed = EnphaseDataService.enphaseData.data[insertAtIndex].wh_consumed;
        let currentInsertIndex = insertAtIndex;
        for (let i = firstImportDataIndex; i <= lastImportDataIndex; i++) {
            let producted = importData[i].wh_producted;
            if (producted > max_wh_producted) {
                producted = max_wh_producted;
            }
            if (producted < min_wh_producted) {
                producted = min_wh_producted;
            }
            let consumed = importData[i].wh_consumed;
            if (consumed > max_wh_consumed) {
                consumed = max_wh_consumed;
            }
            if (consumed < min_wh_consumed) {
                consumed = min_wh_consumed;
            }
            current_wh_exported += importData[i].wh_period_exported;
            total_exported += importData[i].wh_period_exported;
            let obj = { timestamp: importData[i].timestamp, wh_producted: producted, wh_consumed: consumed, wh_exported: current_wh_exported };
            EnphaseDataService.enphaseData.data.splice(currentInsertIndex, 0, obj);
            currentInsertIndex++;
        }
        let str = this.computeImportStrDate(timestamp1, timestamp2, total_exported);
        EnphaseDataService.addDataFromIndex(currentInsertIndex, total_exported);
        return str;
    }
    static addDataFromIndex(index, exportedOffset) {
        for (let i = index; i < EnphaseDataService.enphaseData.data.length; i++) {
            EnphaseDataService.enphaseData.data[i].wh_exported += exportedOffset;
        }
        /*for (let i: number = 1; i<EnphaseDataService.enphaseData.data.length; i++) {
            let difExported = EnphaseDataService.enphaseData.data[i].wh_exported-EnphaseDataService.enphaseData.data[i-1].wh_exported;
            let difProducted = EnphaseDataService.enphaseData.data[i].wh_producted-EnphaseDataService.enphaseData.data[i-1].wh_producted;
            if (difExported>difProducted) {
                EnphaseDataService.enphaseData.data[i].wh_exported = EnphaseDataService.enphaseData.data[i-1].wh_exported + difProducted;
            }
        }
        for (let i: number = 0; i<EnphaseDataService.enphaseData.data_detail.length; i++) {
            if (EnphaseDataService.enphaseData.data_detail[i].timestamp>EnphaseDataService.enphaseData.data[index].timestamp) {
                EnphaseDataService.enphaseData.data_detail[i].wh_exported += exportedOffset;
            }
        }
        for (let i: number = 1; i<EnphaseDataService.enphaseData.data_detail.length; i++) {
            let difExported = EnphaseDataService.enphaseData.data_detail[i].wh_exported-EnphaseDataService.enphaseData.data_detail[i-1].wh_exported;
            let difProducted = EnphaseDataService.enphaseData.data_detail[i].wh_producted-EnphaseDataService.enphaseData.data_detail[i-1].wh_producted;
            if (difExported>difProducted) {
                EnphaseDataService.enphaseData.data_detail[i].wh_exported = EnphaseDataService.enphaseData.data_detail[i-1].wh_exported + difProducted;
            }
        }*/
        for (let i = 0; i < EnphaseDataService.enphaseData.data_detail.length; i++) {
            EnphaseDataService.enphaseData.data_detail[i].wh_exported += exportedOffset;
        }
        if (EnphaseDataService.enphaseData.data_detail.length > 0) {
            EnphaseDataService.lastExportationWhLifetime = EnphaseDataService.enphaseData.data_detail[EnphaseDataService.enphaseData.data_detail.length - 1].wh_exported;
            EnphaseDataService.lastConsomptionWhLifetime = EnphaseDataService.enphaseData.data_detail[EnphaseDataService.enphaseData.data_detail.length - 1].wh_consumed;
            EnphaseDataService.lastProductionWhLifetime = EnphaseDataService.enphaseData.data_detail[EnphaseDataService.enphaseData.data_detail.length - 1].wh_producted;
        }
        else if (EnphaseDataService.enphaseData.data.length > 0) {
            EnphaseDataService.lastExportationWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_exported;
            EnphaseDataService.lastConsomptionWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_consumed;
            EnphaseDataService.lastProductionWhLifetime = EnphaseDataService.enphaseData.data[EnphaseDataService.enphaseData.data.length - 1].wh_producted;
        }
    }
    static computeStrDate(timestamp1, timestamp2) {
        let str = 'Date range to fill ';
        str += new Date(timestamp1 * 1000).toLocaleDateString() + ' ' + new Date(timestamp1 * 1000).toLocaleTimeString() + ' to ' + new Date(timestamp2 * 1000).toLocaleDateString() + ' ' + new Date(timestamp2 * 1000).toLocaleTimeString();
        return str;
    }
    static computeImportStrDate(timestamp1, timestamp2, nbExported) {
        let str = 'Date range to fill ';
        str += new Date(timestamp1 * 1000).toLocaleDateString() + ' ' + new Date(timestamp1 * 1000).toLocaleTimeString() + ' to ' + new Date(timestamp2 * 1000).toLocaleDateString() + ' ' + new Date(timestamp2 * 1000).toLocaleTimeString() + ' -> Exported : ' + nbExported + ' Wh';
        return str;
    }
    static getProductedAtTimestamp(timestamp) {
        let data = EnphaseDataService.enphaseData.data;
        for (let i = 0; i < data.length; i++) {
            if (data[i].timestamp > timestamp) {
                if (i === 0) {
                    return 0;
                }
                let coef = (timestamp - data[i - 1].timestamp) / (data[i].timestamp - data[i - 1].timestamp);
                let result = data[i - 1].wh_producted + coef * (data[i].wh_producted - data[i - 1].wh_producted);
                return result;
            }
        }
        return data[data.length - 1].wh_producted;
    }
    static getExportedAtTimestamp(timestamp) {
        let data = EnphaseDataService.enphaseData.data;
        for (let i = 0; i < data.length; i++) {
            if (data[i].timestamp > timestamp) {
                if (i === 0) {
                    return 0;
                }
                let coef = (timestamp - data[i - 1].timestamp) / (data[i].timestamp - data[i - 1].timestamp);
                let result = data[i - 1].wh_exported + coef * (data[i].wh_exported - data[i - 1].wh_exported);
                return result;
            }
        }
        return data[data.length - 1].wh_exported;
    }
    static getConsumedAtTimestamp(timestamp) {
        let data = EnphaseDataService.enphaseData.data;
        for (let i = 0; i < data.length; i++) {
            if (data[i].timestamp > timestamp) {
                if (i === 0) {
                    return 0;
                }
                let coef = (timestamp - data[i - 1].timestamp) / (data[i].timestamp - data[i - 1].timestamp);
                let result = data[i - 1].wh_consumed + coef * (data[i].wh_consumed - data[i - 1].wh_consumed);
                return result;
            }
        }
        return data[data.length - 1].wh_consumed;
    }
}
exports.EnphaseDataService = EnphaseDataService;
EnphaseDataService.extraDetail = [];
EnphaseDataService.realTimeArray = [];
EnphaseDataService.lastReadingTime = 0;
EnphaseDataService.currentProduction = 0;
EnphaseDataService.currentConsumption = 0;
EnphaseDataService.lastExportationWhLifetime = 0;
EnphaseDataService.lastProductionWhLifetime = 0;
EnphaseDataService.lastConsomptionWhLifetime = 0;
EnphaseDataService.needData = false;
