"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyEnergiService = void 0;
const myenergi_api_1 = require("myenergi-api");
const configuration_service_1 = require("../services/configuration.service");
const enphase_data_service_1 = require("../enphase/enphase.data.service");
class MyEnergiService {
    constructor() {
    }
    static init() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let configuration = configuration_service_1.ConfigurationService.getConfiguration();
            console.log('Init of MyEnergiService');
            this.chargingMode = (_a = configuration.myEnergi) === null || _a === void 0 ? void 0 : _a.chargingMode;
            if ((_b = configuration.myEnergi) === null || _b === void 0 ? void 0 : _b.useMyEnergi) {
                console.log('Use MyEnergy = True => Initialisation de l accés à MyEnergi !');
                this.myEnergi = new myenergi_api_1.MyEnergi(configuration.myEnergi.serialNumber, configuration.myEnergi.token);
            }
        });
    }
    // Permet de mémoriser les données de charge
    static addToChargeData(value) {
        this.chargeData.push(value);
        // Remove too old data
        while (this.chargeData.length > 0 && this.chargeData[0].timestamp < (new Date().getTime() / 1000) - 3600 * 24) {
            this.chargeData.splice(0, 1);
        }
    }
    ;
    // Permet de mémoriser les données de routage
    static addToRouteData(value) {
        this.routeData.push(value);
        // Remove too old data
        while (this.routeData.length > 0 && this.routeData[0].timestamp < (new Date().getTime() / 1000) - 3600 * 24) {
            this.routeData.splice(0, 1);
        }
    }
    ;
    //Ajoute les valeurs de charge récentes dans le fichier enphase_data
    static addChargeValue(enphaseArray) {
        let currentIndex = 0;
        for (let i = 0; i < enphaseArray.length; i++) {
            enphaseArray[i].w_charge = 0;
        }
        for (let i = 1; i < enphaseArray.length; i++) {
            let timestamp = enphaseArray[i - 1].timestamp;
            while (currentIndex < this.chargeData.length && this.chargeData[currentIndex].timestamp < timestamp) {
                currentIndex++;
            }
            if (currentIndex >= this.chargeData.length) {
                return enphaseArray;
            }
            if (this.chargeData[currentIndex].timestamp < enphaseArray[i].timestamp) {
                enphaseArray[i].w_charge = this.chargeData[currentIndex].w_charge;
            }
        }
        return enphaseArray;
    }
    /**
     * Set the myEnregi charging mode
     * @param myEnergiChargingMode
     */
    static setMode(myEnergiChargingMode) {
        MyEnergiService.chargingMode = myEnergiChargingMode;
    }
    /**
     * Obtenir les dernières données de configuration
     */
    static getConfData() {
        this.last_conf_data.chargingMode = this.chargingMode;
        return this.last_conf_data;
    }
    ;
    static getZappiData() {
        let result = {};
        result.zappiData = this.last_zappi_data;
        result.managementData = {};
        result.managementData.startOverflowLake = this.startOverflowLake * 1000;
        result.managementData.timeWithoutExport = this.timeWithoutExport * 1000;
        result.managementData.stopTime = this.stopTime * 1000;
        result.managementData.timeSinceLastStop = this.timeSinceLastStop * 1000;
        result.managementData.canChargeTime = this.canChargeTime * 1000;
        result.managementData.timeSinceCanCharge = this.timeSinceCanCharge * 1000;
        return result;
    }
    //Ajoute les valeurs de routage récentes dans les données enphase_data pour fabriquer les courbes 
    static addRouteValue(enphaseArray) {
        let currentIndex = 0;
        for (let i = 0; i < enphaseArray.length; i++) {
            enphaseArray[i].w_route = 0;
        }
        for (let i = 1; i < enphaseArray.length; i++) {
            let timestamp = enphaseArray[i - 1].timestamp;
            while (currentIndex < this.routeData.length && this.routeData[currentIndex].timestamp < timestamp) {
                currentIndex++;
            }
            if (currentIndex >= this.routeData.length) {
                return enphaseArray;
            }
            if (this.routeData[currentIndex].timestamp < enphaseArray[i].timestamp) {
                enphaseArray[i].w_route = this.routeData[currentIndex].w_route;
            }
        }
        return enphaseArray;
    }
    static getNewData() {
        return __awaiter(this, void 0, void 0, function* () {
            let configuration = yield configuration_service_1.ConfigurationService.getConfiguration();
            // Récupération des données de la zappi
            if (configuration != null && configuration != undefined && configuration.myEnergi != null && configuration.myEnergi != undefined && configuration.myEnergi.serialNumber != null && configuration.myEnergi.serialNumber != undefined) {
                let zappiData = yield this.myEnergi.getStatusZappi(configuration.myEnergi.serialNumber);
                if (zappiData == null) {
                    console.error('Données Zappi null.');
                }
                this.last_zappi_data = zappiData;
                return zappiData;
            }
            return null;
        });
    }
    static checkModeSolarPlus() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        return __awaiter(this, void 0, void 0, function* () {
            let configuration = configuration_service_1.ConfigurationService.getConfiguration();
            let now = enphase_data_service_1.EnphaseDataService.getLastReadingTime();
            let routedValue = 0;
            //console.log('1 :' + this.last_zappi_data?.ectt1 + " : " + this.last_zappi_data?.ectp1);
            //console.log('2 :' + this.last_zappi_data?.ectt2 + " : " + this.last_zappi_data?.ectp2);
            //console.log('3 :' + this.last_zappi_data?.ectt3 + " : " + this.last_zappi_data?.ectp3);
            //console.log('4 :' + this.last_zappi_data?.ectt4 + " : " + this.last_zappi_data?.ectp4);
            //console.log('5 :' + this.last_zappi_data?.ectt5 + " : " + this.last_zappi_data?.ectp5);
            //console.log('6 :' + this.last_zappi_data?.ectt6 + " : " + this.last_zappi_data?.ectp6);
            // Si le tore du routeur est connecté (nom alimenté et valeur numérique)
            if (((_a = this.last_zappi_data) === null || _a === void 0 ? void 0 : _a.ectt4) !== null && !isNaN((_b = this.last_zappi_data) === null || _b === void 0 ? void 0 : _b.ectp4)) {
                routedValue = Math.abs((_c = this.last_zappi_data) === null || _c === void 0 ? void 0 : _c.ectp4);
            }
            if (routedValue > 0) {
                let routeData = { timestamp: now, w_route: routedValue };
                this.addToRouteData(routeData);
            }
            let EvDisconnected = "A";
            let Fault = "F";
            if ((((_d = this.last_zappi_data) === null || _d === void 0 ? void 0 : _d.pst) == EvDisconnected) || (((_e = this.last_zappi_data) === null || _e === void 0 ? void 0 : _e.pst) == Fault)) {
                this.initWorkValue();
                // Si la borne est sur Eco, c'est que la voiture a été débranché sans passage en Off! 
                // Pour éviter une reprise de charge par erreur, passage en Off
                if (((_f = this.last_zappi_data) === null || _f === void 0 ? void 0 : _f.zmo) == myenergi_api_1.ZappiChargeMode.Eco) {
                    this.myEnergi.setZappiChargeMode((_g = this.last_zappi_data) === null || _g === void 0 ? void 0 : _g.sno, myenergi_api_1.ZappiChargeMode.Off);
                }
                return;
            }
            let chargedValue = 0;
            if (!isNaN((_h = this.last_zappi_data) === null || _h === void 0 ? void 0 : _h.ectp1)) {
                chargedValue = (_j = this.last_zappi_data) === null || _j === void 0 ? void 0 : _j.ectp1;
            }
            if (chargedValue > 0) {
                let chargeData = { timestamp: now, w_charge: chargedValue };
                this.addToChargeData(chargeData);
            }
            // Si la charge est en mode normal, alors sortir
            if (this.chargingMode === "normal") {
                this.initWorkValue();
                return;
            }
            // Si la charge a été arrêté, on attend le délai imparti avant de remettre en route
            if (this.stopTime !== 0) {
                this.timeSinceLastStop = now - this.stopTime;
                if (this.timeSinceLastStop <= configuration.myEnergi.restartDelay) {
                    return;
                }
                this.stopTime = 0;
                this.timeSinceLastStop = 0;
            }
            let canCharge = this.canCharge(routedValue, chargedValue, configuration);
            // Si la charge n'est pas active, on essaye de l'activer mais on prend un délai pour éviter de prendre en compte l'effet Albedo
            if (((_k = this.last_zappi_data) === null || _k === void 0 ? void 0 : _k.zmo) == myenergi_api_1.ZappiChargeMode.Off) {
                this.checkOffMode(canCharge, now, configuration, this.last_zappi_data);
                return;
            }
            // Si la charge est active, on vérifie que les conditions sont toujours remplies
            if (((_l = this.last_zappi_data) === null || _l === void 0 ? void 0 : _l.zmo) == myenergi_api_1.ZappiChargeMode.Eco) {
                this.checkModeEco(canCharge, now, configuration, this.last_zappi_data);
                return;
            }
        });
    }
    static initWorkValue() {
        this.stopTime = 0;
        this.timeSinceLastStop = 0;
        this.startOverflowLake = 0;
        this.timeWithoutExport = 0;
        this.canChargeTime = 0;
        this.timeSinceCanCharge = 0;
    }
    static canCharge(routedValue, chargedValue, configuration) {
        var _a, _b;
        let theoricalExport = 0;
        //console.log('Can charge ? '); 
        //console.log('1 :' + this.last_zappi_data?.ectt1 + " : " + this.last_zappi_data?.ectp1);
        //console.log('2 :' + this.last_zappi_data?.ectt2 + " : " + this.last_zappi_data?.ectp2);
        //console.log('3 :' + this.last_zappi_data?.ectt3 + " : " + this.last_zappi_data?.ectp3);
        //console.log('4 :' + this.last_zappi_data?.ectt4 + " : " + this.last_zappi_data?.ectp4);
        //console.log('5 :' + this.last_zappi_data?.ectt5 + " : " + this.last_zappi_data?.ectp5);
        //console.log('6 :' + this.last_zappi_data?.ectt6 + " : " + this.last_zappi_data?.ectp6);
        //console.log('routed value : ' + routedValue);
        // Le tor de mesure de routeur est présent si son nom est alimenté ;)
        if ((((_a = this.last_zappi_data) === null || _a === void 0 ? void 0 : _a.ectt4) !== null || ((_b = this.last_zappi_data) === null || _b === void 0 ? void 0 : _b.ectt4) !== "None")) {
            //console.log('Troisième tore présent ! ');
            // Calcul d'un export réél en réintégrant le routage et la charge de la voiture 
            theoricalExport = enphase_data_service_1.EnphaseDataService.currentProduction - enphase_data_service_1.EnphaseDataService.currentConsumption + routedValue + chargedValue;
            //console.log('3ème tore surplus =  ' + theoricalExport);
        }
        else {
            //console.log('Sans tore présent pour le routeur! ');
            // Calcul d'un export théorique sur la base du bruit de fond déclaré dans la configuration
            theoricalExport = enphase_data_service_1.EnphaseDataService.currentProduction - configuration.myEnergi.backgroundNoise;
            //console.log('Surplus théorique =  ' + theoricalExport);
        }
        // On peut charger si on a de l'export et que la production est supérieure à la consommation - une marge d'erreur (fluctuation du routeur et du chargeur)
        //console.log('theoricalExport :  ' + theoricalExport);
        //console.log('theoricalExportLimit :  ' + configuration.myEnergi.theoricalExportLimit);
        //console.log('currentProduction :  ' + EnphaseDataService.currentProduction);
        //console.log('currentConsumption :  ' + EnphaseDataService.currentConsumption);
        //console.log('adjustmentMargin :  ' + configuration.myEnergi.adjustmentMargin);
        //console.log('Test limite Export :' + theoricalExport >= configuration.myEnergi.theoricalExportLimit);
        //console.log('Test Consommation  : ' + (Math.abs(EnphaseDataService.currentProduction - EnphaseDataService.currentConsumption) <= configuration.myEnergi.adjustmentMargin));
        //console.log('Consommation résiduelle : ' + (Math.abs(EnphaseDataService.currentProduction - EnphaseDataService.currentConsumption)));
        let canCharge = (theoricalExport >= configuration.myEnergi.theoricalExportLimit)
            && Math.abs(enphase_data_service_1.EnphaseDataService.currentProduction - enphase_data_service_1.EnphaseDataService.currentConsumption) <= configuration.myEnergi.adjustmentMargin;
        //console.log('Peu chargé ? ' + canCharge);    
        return canCharge;
    }
    static checkOffMode(canCharge, now, configuration, zappiData) {
        if (!canCharge) {
            this.startOverflowLake = 0;
            this.timeWithoutExport = 0;
            this.canChargeTime = 0;
            this.timeSinceCanCharge = 0;
            return;
        }
        if (this.canChargeTime == 0) {
            this.canChargeTime = now;
            return;
        }
        this.timeSinceCanCharge = now - this.canChargeTime;
        if (this.timeSinceCanCharge <= configuration.myEnergi.startDelay) {
            return;
        }
        // Les conditions sont remplies, on peut charger
        this.myEnergi.setZappiChargeMode(zappiData.sno, myenergi_api_1.ZappiChargeMode.Eco);
        this.canChargeTime = 0;
        this.timeSinceCanCharge = 0;
        return;
    }
    static checkModeEco(canCharge, now, configuration, zappiData) {
        if (canCharge) {
            this.startOverflowLake = 0;
            this.timeWithoutExport = 0;
            this.canChargeTime = 0;
            this.timeSinceCanCharge = 0;
            return;
        }
        //Si le manque de surplus n'est pas encore détecté, on le marque pour le délai
        if (this.startOverflowLake == 0) {
            this.startOverflowLake = now;
            return;
        }
        // Le manque de surplus est déjà connue, on vérifie si le délai est dépassé ou pas
        this.timeWithoutExport = now - this.startOverflowLake;
        if (this.timeWithoutExport >= configuration.myEnergi.stopDelay) {
            this.myEnergi.setZappiChargeMode(zappiData.sno, myenergi_api_1.ZappiChargeMode.Off);
            this.startOverflowLake = 0;
            this.timeWithoutExport = 0;
            this.stopTime = now;
            return;
        }
    }
}
exports.MyEnergiService = MyEnergiService;
MyEnergiService.chargeData = [];
MyEnergiService.routeData = [];
MyEnergiService.startOverflowLake = 0;
MyEnergiService.timeWithoutExport = 0;
MyEnergiService.stopTime = 0;
MyEnergiService.canChargeTime = 0;
MyEnergiService.timeSinceCanCharge = 0;
MyEnergiService.chargingMode = "normal";
MyEnergiService.last_conf_data = {};
MyEnergiService.last_zappi_data = {};
