"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/** source/server.ts */
const http_1 = __importDefault(require("http"));
const express_1 = __importDefault(require("express"));
const morgan_1 = __importDefault(require("morgan"));
const router_1 = __importDefault(require("./routes/router"));
const enphase_data_service_1 = require("./enphase/enphase.data.service");
const path_1 = __importDefault(require("path"));
const tesla_service_1 = require("./tesla/tesla.service");
const configuration_service_1 = require("./services/configuration.service");
const myEnergi_service_1 = require("./myEnergi/myEnergi.service");
const router = (0, express_1.default)();
/** Logging */
router.use((0, morgan_1.default)('dev'));
/** Parse the request */
router.use(express_1.default.urlencoded({ extended: false }));
/** Takes care of JSON data */
//router.use(express.json());
/** Takes care of JSON data */
router.use('/assets', express_1.default.static(path_1.default.join()));
router.use(express_1.default.json({ limit: '10mb' }));
router.use(express_1.default.urlencoded({ limit: '10mb', extended: true }));
/** RULES OF OUR API */
router.use((req, res, next) => {
    // set the CORS policy
    res.header('Access-Control-Allow-Origin', '*');
    // set the CORS headers
    res.header('Access-Control-Allow-Headers', 'origin, X-Requested-With,Content-Type,Accept, Authorization');
    // set the CORS method headers
    if (req.method === 'OPTIONS') {
        res.header('Access-Control-Allow-Methods', 'GET POST');
        return res.status(200).json({});
    }
    next();
});
/** Routes */
router.use('/', router_1.default);
/** Error handling */
router.use((req, res, next) => {
    const error = new Error('not found');
    return res.status(404).json({
        message: error.message
    });
});
run();
function run() {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        /** Init */
        yield configuration_service_1.ConfigurationService.init();
        yield enphase_data_service_1.EnphaseDataService.init();
        yield tesla_service_1.TeslaService.init();
        yield myEnergi_service_1.MyEnergiService.init();
        /** Server */
        const httpServer = http_1.default.createServer(router);
        const PORT = (_a = process.env.PORT) !== null && _a !== void 0 ? _a : 8002;
        httpServer.listen(PORT, () => console.log(`Le serveur v0.9.40 est en écoute sur le port ${PORT}\r\nVous pouvez maintenant lancer un navigateur avec l'url "localhost:${PORT}"`));
    });
}
