"use strict";
/* exported TeslaUtility */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeslaService = void 0;
const axios_1 = __importDefault(require("axios"));
const enphase_data_service_1 = require("../enphase/enphase.data.service");
const configuration_service_1 = require("../services/configuration.service");
class TeslaService {
    /**
     * Essai de se connecter a l'API et si réussi alors positionne le token et appel la callback
     */
    static connect() {
        if (configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla && configuration_service_1.ConfigurationService.getConfiguration().tesla.useBleProxy) {
            TeslaService.mode = TeslaService.MODE_GET_DATA;
            TeslaService.step = TeslaService.STEP_GET_DATA;
            return;
        }
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 10 * 1000; // On reessaie dans 10 secondes si on n'obtient pas de réponse
        var obj = {};
        obj.grant_type = "refresh_token";
        obj.client_id = "ownerapi";
        obj.refresh_token = configuration_service_1.ConfigurationService.getConfiguration().tesla.token;
        obj.scope = "openid email offline_access";
        axios_1.default.post('https://auth.tesla.com/oauth2/v3/token', obj).then(response => {
            TeslaService.log("connectToVehicle Receive status " + response.status);
            TeslaService.log("connectToVehicle response (" + response.data + ")");
            if (response.status === 200) {
                TeslaService.access_token = response.data.access_token;
                //that.refresh_token = JSON.parse(xhttp.responseText).refresh_token;
                let d2 = new Date();
                TeslaService.next_action_time = d2.getTime() + 0 * 1000; // Prochaine action dans 1 seconde
                if (TeslaService.mode === TeslaService.MODE_REFRESH_TOKEN) {
                    TeslaService.mode = TeslaService.previousMode;
                    TeslaService.step = TeslaService.previousStep;
                    if (TeslaService.mode === TeslaService.MODE_REFRESH_TOKEN) {
                        TeslaService.mode = TeslaService.MODE_GET_DATA;
                        TeslaService.step = TeslaService.STEP_GET_DATA;
                    }
                }
                else {
                    TeslaService.step = TeslaService.STEP_GET_VEHICLES;
                }
            }
            else {
            }
        }).catch(error => {
            console.error('Error axios to get token from tesla');
        });
    }
    ;
    /**
     * Obtenir la liste des vehicles
     */
    static getVehicleList() {
        'use strict';
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 10 * 1000; // On reessaie dans 10 secondes si on n'obtient pas de réponse
        axios_1.default.get('https://owner-api.teslamotors.com/api/1/vehicles', { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            TeslaService.log("getVehicleList Receive status " + response.status);
            let d2 = new Date();
            if (response.status === 200) {
                let vehicle_list = response.data.response;
                TeslaService.vehicle_id = vehicle_list[0].id_s;
                TeslaService.log("vin = " + vehicle_list[0].vin);
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = d2.getTime(); // next step immediatly
            }
            else if (response.status === 401 || response.status === 404) {
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime(); // next step immediatly
            }
        }).catch(error => {
            console.error('Error axios to get vehicle list from tesla');
        });
    }
    ;
    /**
     * Sort la voiture du mode veille
     */
    static wakeUpVehicle() {
        'use strict';
        var _a;
        if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
            let d2 = new Date();
            TeslaService.next_action_time = d2.getTime();
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.nbWakeUp = 0;
            return;
        }
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 30 * 1000; // On reessaie dans 30 secondes si on n'obtient pas de réponse
        axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/wake_up', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            TeslaService.log("wakeUpVehicle Receive status " + response.status);
            let d2 = new Date();
            if (response.status === 200) {
                TeslaService.next_action_time = d2.getTime();
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.nbWakeUp = 0;
            }
            else if (response.status === 401) {
                if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                    TeslaService.previousMode = TeslaService.mode;
                }
                TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                TeslaService.previousStep = TeslaService.step;
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime(); // Prochaine action dans 1 seconde
                TeslaService.nbWakeUp++;
            }
            else if (response.status === 404) {
                TeslaService.mode = TeslaService.MODE_INIT;
                TeslaService.previousStep = TeslaService.step;
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime(); // Prochaine action dans 1 seconde
                TeslaService.nbWakeUp++;
            }
            else {
                TeslaService.nbWakeUp++;
            }
            if (TeslaService.nbWakeUp > 2) {
                TeslaService.mode = TeslaService.MODE_INIT;
                TeslaService.previousStep = TeslaService.step;
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime(); // Prochaine action dans 1 seconde
            }
        }).catch(error => {
            console.error('Error axios to wake up vehicle');
        });
    }
    ;
    /**
     * Sort la voiture du mode veille
     */
    static getOnline() {
        'use strict';
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 10 * 1000; // On reessaie dans 10 secondes si on n'obtient pas de réponse
        axios_1.default.get('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            let d2 = new Date();
            TeslaService.log("getOnline Receive status " + response.status + " - " + (TeslaService.last_success_get_data + TeslaService.max_time_between_get_data - d2.getTime()));
            if (response.status === 200) {
                let d2 = new Date();
                let response2 = response.data.response;
                if (response2.state === "online") {
                    TeslaService.step = TeslaService.MODE_GET_DATA;
                    TeslaService.next_action_time = d2.getTime();
                }
                else if (d2.getTime() > TeslaService.last_success_get_data + TeslaService.max_time_between_get_data) {
                    TeslaService.step = TeslaService.STEP_WAKE_UP;
                    TeslaService.next_action_time = d2.getTime();
                }
            }
            else if (response.status === 401) {
                if (TeslaService.previousMode !== TeslaService.MODE_REFRESH_TOKEN) {
                    TeslaService.previousMode = TeslaService.mode;
                }
                TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                TeslaService.previousStep = TeslaService.step;
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime();
            }
            else if (response.status === 404) {
                if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                    TeslaService.previousMode = TeslaService.mode;
                }
                TeslaService.mode = TeslaService.MODE_INIT;
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = d2.getTime();
            }
        }).catch(error => {
            console.error('Error axios to get vehicle state online');
        });
    }
    ;
    /**
     * Get data of the vehicle (and wake up if necessary)
     */
    static getVehicleData() {
        'use strict';
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (!((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useTesla)) {
            return;
        }
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 10 * 1000; // On reessaie dans 10 secondes si on n'obtient pas de réponse
        if ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.useBleProxy) {
            let d3 = new Date();
            TeslaService.next_action_time = d3.getTime() + 10 * 1000;
            if (TeslaService.mode === TeslaService.MODE_WAIT_ONLINE) {
                try {
                    axios_1.default.get('http://' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyUrl) + ':' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.bleProxyPort) + '/api/1/vehicles/' + ((_e = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _e === void 0 ? void 0 : _e.vin) + '/body_controller_state', {
                        headers: {
                            Accept: "application/json",
                            "User-Agent": "axios 0.21.1"
                        }
                    }).then(response => {
                        try {
                            if (response.status === 200) {
                                if (response.data.response.response.vehicleSleepStatus === "VEHICLE_SLEEP_STATUS_AWAKE") {
                                    let d4 = new Date();
                                    TeslaService.next_action_time = d4.getTime();
                                    TeslaService.mode = TeslaService.MODE_GET_DATA;
                                }
                            }
                        }
                        catch (error) {
                            console.error('Error to get body from tesla');
                        }
                    });
                }
                catch (error) {
                    console.error('Error axios to get body from tesla');
                }
            }
            else {
                axios_1.default.get('http://' + ((_f = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _f === void 0 ? void 0 : _f.bleProxyUrl) + ':' + ((_g = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _g === void 0 ? void 0 : _g.bleProxyPort) + '/api/1/vehicles/' + ((_h = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _h === void 0 ? void 0 : _h.vin) + '/vehicle_data').then(response => {
                    console.log("Receive vehicle data from tesla");
                    TeslaService.log("getVehicleData Receive status " + response.status);
                    let d2 = new Date();
                    try {
                        if (response.status === 200) {
                            let vehicleData = response.data.response.response;
                            vehicleData.timestampMs = d2.getTime();
                            TeslaService.last_receive_data = vehicleData;
                            TeslaService.last_success_get_data = d2.getTime();
                            switch (TeslaService.mode) {
                                case TeslaService.MODE_INIT:
                                case TeslaService.MODE_GET_DATA:
                                case TeslaService.MODE_GET_DATA_NOW:
                                case TeslaService.MODE_WAIT_ONLINE:
                                case TeslaService.MODE_REFRESH_TOKEN:
                                    TeslaService.mode = TeslaService.MODE_WAIT_ONLINE;
                                    TeslaService.next_action_time = d2.getTime() + 20 * 60 * 1000; // On redemande dans 20mn pour laisser le temps à la voiture de se mettre en veille
                                    if (vehicleData.charge_state.charging_state === 'Starting' || vehicleData.charge_state.charging_state === 'Charging') {
                                        TeslaService.mode = TeslaService.MODE_GET_DATA;
                                        TeslaService.next_action_time = d2.getTime() + 10 * 1000; // Si la voiture est en charge, on demande les données toute les 10 secondes
                                        TeslaService.checkChargingMode();
                                    }
                                    else if (vehicleData.drive_state !== null && vehicleData.drive_state != undefined && vehicleData.drive_state.shift_state !== null) {
                                        TeslaService.mode = TeslaService.MODE_GET_DATA;
                                        TeslaService.next_action_time = d2.getTime() + 60 * 1000; // Si le vehicule roule, on demande les données toute les minutes
                                    }
                                    if (vehicleData.charge_state.charging_state !== "Charging") {
                                        if (TeslaService.wasInCharge) {
                                            // If end of charging then set charge to max amp if in solar or solar auto mode
                                            if (TeslaService.chargingMode !== 'normal' && vehicleData.charge_state.charge_amps !== configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps) {
                                                console.log('End of charging so set Tesla to amps ' + configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps);
                                                TeslaService.setChargeAmps(configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps);
                                            }
                                            TeslaService.wasInCharge = false;
                                        }
                                    }
                                    else {
                                        TeslaService.wasInCharge = true;
                                    }
                                    break;
                                case TeslaService.MODE_GET_CONTINUS_DATA:
                                    TeslaService.next_action_time = d2.getTime(); // On redemande aussitot que possible
                                    break;
                            }
                            let power = vehicleData.charge_state.charger_actual_current * vehicleData.charge_state.charger_voltage * vehicleData.charge_state.charger_phases;
                            let chargeData = { timestamp: enphase_data_service_1.EnphaseDataService.getLastReadingTime(), w_charge: power };
                            TeslaService.addToChargeData(chargeData);
                        }
                        else if (response.status === 401) {
                            if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                                TeslaService.previousMode = TeslaService.mode;
                            }
                            TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                            TeslaService.previousStep = TeslaService.step;
                            TeslaService.step = TeslaService.STEP_CONNECT;
                            TeslaService.next_action_time = d2.getTime();
                        }
                        else if (response.status === 404) {
                            if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                                TeslaService.previousMode = TeslaService.mode;
                            }
                            TeslaService.mode = TeslaService.MODE_INIT;
                            TeslaService.step = TeslaService.STEP_CONNECT;
                            TeslaService.next_action_time = d2.getTime();
                        }
                    }
                    catch (error) {
                        console.error('Error to get data from tesla' + error);
                    }
                }).catch((error) => {
                    let d2 = new Date();
                    if (error.response) {
                        if (error.response.status === 401) {
                            if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                                TeslaService.previousMode = TeslaService.mode;
                            }
                            TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                            TeslaService.previousStep = TeslaService.step;
                            TeslaService.step = TeslaService.STEP_CONNECT;
                            TeslaService.next_action_time = d2.getTime();
                        }
                        else if (error.response.status === 404) {
                            if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                                TeslaService.previousMode = TeslaService.mode;
                            }
                            TeslaService.mode = TeslaService.MODE_INIT;
                            TeslaService.step = TeslaService.STEP_CONNECT;
                            TeslaService.next_action_time = d2.getTime();
                        }
                        else {
                            console.error('Error getVehicleData : ' + error.message);
                        }
                    }
                    else {
                        console.error('Error getVehicleData : ' + error.message);
                    }
                });
                return;
            }
        }
        else {
            axios_1.default.get('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/vehicle_data', { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
                TeslaService.log("getVehicleData Receive status " + response.status);
                let d2 = new Date();
                try {
                    if (response.status === 200) {
                        let vehicleData = response.data.response;
                        vehicleData.timestampMs = d2.getTime();
                        TeslaService.last_receive_data = vehicleData;
                        TeslaService.last_success_get_data = d2.getTime();
                        switch (TeslaService.mode) {
                            case TeslaService.MODE_INIT:
                            case TeslaService.MODE_GET_DATA:
                            case TeslaService.MODE_GET_DATA_NOW:
                            case TeslaService.MODE_WAIT_ONLINE:
                            case TeslaService.MODE_REFRESH_TOKEN:
                                TeslaService.mode = TeslaService.MODE_WAIT_ONLINE;
                                TeslaService.next_action_time = d2.getTime() + 20 * 60 * 1000; // On redemande dans 20mn pour laisser le temps à la voiture de se mettre en veille
                                if (vehicleData.charge_state.charging_state === 'Starting' || vehicleData.charge_state.charging_state === 'Charging') {
                                    TeslaService.next_action_time = d2.getTime() + 10 * 1000; // Si la voiture est en charge, on demande les données toute les 10 secondes
                                    TeslaService.checkChargingMode();
                                }
                                else if (vehicleData.drive_state.shift_state !== null) {
                                    TeslaService.next_action_time = d2.getTime() + 60 * 1000; // Si le vehicule roule, on demande les données toute les minutes
                                }
                                if (vehicleData.charge_state.charging_state !== "Charging") {
                                    if (TeslaService.wasInCharge) {
                                        // If end of charging then set charge to max amp if in solar or solar auto mode
                                        if (TeslaService.chargingMode !== 'normal' && vehicleData.charge_state.charge_amps !== configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps) {
                                            console.log('End of charging so set Tesla to amps ' + configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps);
                                            TeslaService.setChargeAmps(configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps);
                                        }
                                        TeslaService.wasInCharge = false;
                                    }
                                }
                                else {
                                    TeslaService.wasInCharge = true;
                                }
                                break;
                            case TeslaService.MODE_GET_CONTINUS_DATA:
                                TeslaService.next_action_time = d2.getTime(); // On redemande aussitot que possible
                                break;
                        }
                        let power = vehicleData.charge_state.charger_actual_current * vehicleData.charge_state.charger_voltage * vehicleData.charge_state.charger_phases;
                        let chargeData = { timestamp: enphase_data_service_1.EnphaseDataService.getLastReadingTime(), w_charge: power };
                        TeslaService.addToChargeData(chargeData);
                    }
                    else if (response.status === 401) {
                        if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                            TeslaService.previousMode = TeslaService.mode;
                        }
                        TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                        TeslaService.previousStep = TeslaService.step;
                        TeslaService.step = TeslaService.STEP_CONNECT;
                        TeslaService.next_action_time = d2.getTime();
                    }
                    else if (response.status === 404) {
                        if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                            TeslaService.previousMode = TeslaService.mode;
                        }
                        TeslaService.mode = TeslaService.MODE_INIT;
                        TeslaService.step = TeslaService.STEP_CONNECT;
                        TeslaService.next_action_time = d2.getTime();
                    }
                }
                catch (error) {
                    console.error('Error to get data from tesla');
                }
            }).catch((error) => {
                let d2 = new Date();
                if (error.response) {
                    if (error.response.status === 401) {
                        if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                            TeslaService.previousMode = TeslaService.mode;
                        }
                        TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
                        TeslaService.previousStep = TeslaService.step;
                        TeslaService.step = TeslaService.STEP_CONNECT;
                        TeslaService.next_action_time = d2.getTime();
                    }
                    else if (error.response.status === 404) {
                        if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                            TeslaService.previousMode = TeslaService.mode;
                        }
                        TeslaService.mode = TeslaService.MODE_INIT;
                        TeslaService.step = TeslaService.STEP_CONNECT;
                        TeslaService.next_action_time = d2.getTime();
                    }
                    else {
                        console.error('Error getVehicleData : ' + error.message);
                    }
                }
                else {
                    console.error('Error getVehicleData : ' + error.message);
                }
            });
        }
    }
    ;
    static addToChargeData(value) {
        TeslaService.chargeData.push(value);
        // Remove too old data
        while (TeslaService.chargeData.length > 0 && TeslaService.chargeData[0].timestamp < (new Date().getTime() / 1000) - 3600 * 24) {
            TeslaService.chargeData.splice(0, 1);
        }
    }
    ;
    /**
     * Demande un rafraichissement des données immediat
     */
    static getImmediateVehicleData() {
        let d = new Date();
        TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
        TeslaService.step = TeslaService.STEP_WAKE_UP;
        TeslaService.next_action_time = d.getTime();
    }
    ;
    /**
     * Set as continuous mode
     */
    static setContinuousMode() {
        let d = new Date();
        TeslaService.mode = TeslaService.MODE_GET_CONTINUS_DATA;
        if (TeslaService.step === TeslaService.STEP_GET_ONLINE) {
            TeslaService.step = TeslaService.STEP_WAKE_UP;
        }
        else {
            TeslaService.step = TeslaService.STEP_GET_DATA;
        }
        TeslaService.next_action_time = d.getTime();
    }
    ;
    /**
     * Set as continuous mode
     */
    static setNormalMode() {
        let d = new Date();
        TeslaService.mode = TeslaService.MODE_GET_DATA;
        TeslaService.step = TeslaService.STEP_GET_DATA;
        TeslaService.next_action_time = d.getTime();
    }
    ;
    /**
     * La montre passe en mode affichage de l'heure donc on reprend la lecture de l'état en mode normal
     */
    static displayScreenHour() {
        if (TeslaService.mode === TeslaService.MODE_GET_CONTINUS_DATA) {
            TeslaService.setNormalMode();
        }
    }
    ;
    /**
     * Ouvrir le frunk.
     */
    static frunkOpen() {
        let xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                var vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Capot avant ouvert");
                }
            }
        };
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/actuate_trunk");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send("{\"which_trunk\":\"front\"}");
    }
    ;
    /**
     * Ouvrir le port de charge ou débloquer la prise si déjà ouvert.
     */
    static openChargeDoor() {
        let xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                let vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Trappe ouverte");
                }
            }
        };
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/charge_port_door_open");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send();
    }
    ;
    /**
     * Fermer le port de charge.
     */
    static closeChargeDoor() {
        let xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                var vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Trappe fermée");
                }
            }
        };
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/charge_port_door_close");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send();
    }
    ;
    /**
     * Verrouiller les portes.
     */
    static doorLock() {
        let xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                let vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Portes verrouillées");
                }
            }
        };
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/door_lock");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send();
    }
    ;
    /**
     * Fermer les fenêtres.
     */
    static ventClose() {
        let xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                let vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Fenêtres fermées");
                }
            }
        };
        var obj = {};
        obj.command = "close";
        obj.lat = 0;
        obj.long = 0;
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/window_control");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send(JSON.stringify(obj));
    }
    ;
    /**
     * Démarre la charge.
     */
    static startCharge() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
                try {
                    let response = yield axios_1.default.post('http://' + ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.bleProxyUrl) + ':' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyPort) + '/api/1/vehicles/' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.vin) + '/command/charge_start', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } });
                    if (response) {
                        let d2 = new Date();
                        if (response.status === 200) {
                            TeslaService.next_action_time = new Date().getTime() + 2000;
                            TeslaService.step = TeslaService.STEP_GET_DATA;
                            TeslaService.nbWakeUp = 0;
                            return true;
                        }
                        else {
                            TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                            TeslaService.step = TeslaService.STEP_WAKE_UP;
                            TeslaService.next_action_time = new Date().getTime();
                        }
                    }
                }
                catch (error) {
                    console.error('Error axios to start charge from tesla');
                    TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                    TeslaService.step = TeslaService.STEP_WAKE_UP;
                    TeslaService.next_action_time = new Date().getTime();
                    TeslaService.wakeUpVehicle();
                }
                ;
                return false;
            }
            try {
                let response = yield axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/command/charge_start', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } });
                if (response) {
                    let d2 = new Date();
                    if (response.status === 200) {
                        TeslaService.next_action_time = new Date().getTime() + 2000;
                        TeslaService.step = TeslaService.STEP_GET_DATA;
                        TeslaService.nbWakeUp = 0;
                        return true;
                    }
                    else {
                        TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                        TeslaService.step = TeslaService.STEP_WAKE_UP;
                        TeslaService.next_action_time = new Date().getTime();
                    }
                }
            }
            catch (error) {
                console.error('Error axios to start charge from tesla');
                TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = new Date().getTime();
                TeslaService.wakeUpVehicle();
            }
            ;
            return false;
        });
    }
    ;
    /**
     * Arrêter la charge.
     */
    static stopCharge() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
                try {
                    let response = yield axios_1.default.post('http://' + ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.bleProxyUrl) + ':' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyPort) + '/api/1/vehicles/' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.vin) + '/command/charge_stop', null, { 'headers': { 'Content-Type': 'application/json' } });
                    if (response) {
                        let d2 = new Date();
                        if (response.status === 200) {
                            TeslaService.next_action_time = new Date().getTime() + 2000;
                            TeslaService.step = TeslaService.STEP_GET_DATA;
                            TeslaService.nbWakeUp = 0;
                            return true;
                        }
                        else {
                            TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                            TeslaService.step = TeslaService.STEP_WAKE_UP;
                            TeslaService.next_action_time = new Date().getTime();
                        }
                    }
                }
                catch (error) {
                    console.error('Error axios to stop charge from tesla');
                    TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                    TeslaService.step = TeslaService.STEP_WAKE_UP;
                    TeslaService.next_action_time = new Date().getTime();
                }
                ;
            }
            try {
                let response = yield axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/command/charge_stop', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } });
                if (response) {
                    let d2 = new Date();
                    if (response.status === 200) {
                        TeslaService.next_action_time = new Date().getTime() + 2000;
                        TeslaService.step = TeslaService.STEP_GET_DATA;
                        TeslaService.nbWakeUp = 0;
                        return true;
                    }
                    else {
                        TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                        TeslaService.step = TeslaService.STEP_WAKE_UP;
                        TeslaService.next_action_time = new Date().getTime();
                    }
                }
            }
            catch (error) {
                console.error('Error axios to stop charge from tesla');
                TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = new Date().getTime();
            }
            ;
            return false;
        });
    }
    ;
    /**
     * Démarre la climatisation.
     */
    static startClimatisation() {
        var _a, _b, _c, _d;
        if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
            axios_1.default.post('http://' + ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.bleProxyUrl) + ':' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyPort) + '/api/1/vehicles/' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.vin) + '/command/auto_conditioning_start').then(response => {
                TeslaService.log("start climatisation Receive status " + response.status);
                let d2 = new Date();
                if (response.status === 200) {
                    TeslaService.next_action_time = d2.getTime();
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.nbWakeUp = 0;
                }
            }).catch((error) => {
                console.error('Error axios to start climatisation from tesla');
            });
            return;
        }
        axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/command/auto_conditioning_start', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            TeslaService.log("start climatisation Receive status " + response.status);
            let d2 = new Date();
            if (response.status === 200) {
                TeslaService.next_action_time = d2.getTime();
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.nbWakeUp = 0;
            }
        }).catch(error => {
            console.error('Error axios to start climatisation from tesla');
        });
    }
    ;
    /**
     * Stop la climatisation.
     */
    static stopClimatisation() {
        var _a, _b, _c, _d;
        if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
            axios_1.default.post('http://' + ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.bleProxyUrl) + ':' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyPort) + '/api/1/vehicles/' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.vin) + '/command/auto_conditioning_stop').then(response => {
                TeslaService.log("stop climatisation Receive status " + response.status);
                let d2 = new Date();
                if (response.status === 200) {
                    TeslaService.next_action_time = d2.getTime();
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.nbWakeUp = 0;
                }
            }).catch((error) => {
                console.log("Error axios to stop climatisation from tesla " + error);
            });
            return;
        }
        axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/command/auto_conditioning_stop', null, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            TeslaService.log("stop climatisation Receive status " + response.status);
            let d2 = new Date();
            if (response.status === 200) {
                TeslaService.next_action_time = d2.getTime();
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.nbWakeUp = 0;
            }
        }).catch(error => {
            console.error('Error axios to stop climatisation from tesla');
        });
    }
    ;
    /**
     * Démarre le mode sentinel.
     */
    static startSentinel() {
        var xhr = new XMLHttpRequest();
        xhr.onload = function () {
            TeslaService.log("reponse recu de tesla : " + this.responseText);
            if (this.status === 200) {
                let vehicleResponse = JSON.parse(this.responseText).response;
                if (vehicleResponse.result) {
                    TeslaService.log("Sentinel démarrée");
                }
            }
        };
        xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/set_sentry_mode");
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
        xhr.send("{\"on\":true}");
    }
    ;
    static setChargeAmps(amps) {
        var _a, _b, _c, _d;
        if (amps < 5 || amps > 32) {
            return;
        }
        let obj = {
            charging_amps: amps
        };
        if ((_a = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _a === void 0 ? void 0 : _a.useBleProxy) {
            axios_1.default.post('http://' + ((_b = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _b === void 0 ? void 0 : _b.bleProxyUrl) + ':' + ((_c = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _c === void 0 ? void 0 : _c.bleProxyPort) + '/api/1/vehicles/' + ((_d = configuration_service_1.ConfigurationService.getConfiguration().tesla) === null || _d === void 0 ? void 0 : _d.vin) + '/command/set_charging_amps', obj, { 'headers': { 'Content-Type': 'application/json' } }).then(response => {
                TeslaService.log("set charging amps receive status " + response.status);
            }).catch((error) => {
                console.log("Error axios to set charge amps from tesla" + error);
            });
            return;
        }
        axios_1.default.post('https://owner-api.teslamotors.com/api/1/vehicles/' + TeslaService.vehicle_id + '/command/set_charging_amps', obj, { 'headers': { 'Authorization': 'Bearer ' + TeslaService.access_token, 'Content-Type': 'application/json' } }).then(response => {
            TeslaService.log("set charging amps receive status " + response.status);
        }).catch(error => {
            console.error('Error axios to set charge amps from tesla');
        });
    }
    /**
     * Obtenir les dernières données reçues sur le véhicule (avec le timestampMs qui indique quand ces données sont arrivées)
     */
    static getLastReceiveData() {
        TeslaService.last_receive_data.chargingMode = TeslaService.chargingMode;
        return TeslaService.last_receive_data;
    }
    ;
    /**
     * Remove bad caracteres of a token
     */
    static removeTokenBadCaracteres(token) {
        var newToken = "";
        for (let i = 0; i < token.length; i++) {
            if (token.charAt(i) != String.fromCharCode(10) && token.charAt(i) != String.fromCharCode(13) && token.charAt(i) != " ") {
                newToken += token.charAt(i);
            }
        }
        return newToken;
    }
    /**
     * Log information
     */
    static log(text) {
        let d = new Date();
        console.log(d.toLocaleTimeString() + " - " + text);
    }
    ;
    /**
     * Set the tesla charging mode
     * @param teslaChargingMode
     */
    static setMode(teslaChargingMode) {
        TeslaService.chargingMode = teslaChargingMode;
    }
    /**
     * Check if charging power must be modified
     */
    static checkChargingMode() {
        if (TeslaService.chargingMode === 'normal') {
            return;
        }
        if (!TeslaService.isTeslaAtHome()) {
            console.log('Don\'t optimize charge because tesla not at home');
            return;
        }
        if (TeslaService.lastChangeChargingAmps < new Date().getTime() - configuration_service_1.ConfigurationService.getConfiguration().tesla.adaptChargeFrequency * 1000) {
            let overProduction = enphase_data_service_1.EnphaseDataService.getOverproduction(configuration_service_1.ConfigurationService.getConfiguration().tesla.adaptChargeFrequency);
            if (overProduction === null) {
                return;
            }
            console.log('Overproduction : ' + overProduction);
            let newAmps = TeslaService.last_receive_data.charge_state.charge_amps;
            let marginCharge = configuration_service_1.ConfigurationService.getConfiguration().tesla.chargeOffset;
            let ampsDelta = Math.round((overProduction + marginCharge) / TeslaService.last_receive_data.charge_state.charger_voltage / TeslaService.last_receive_data.charge_state.charger_phases);
            newAmps += ampsDelta;
            if (newAmps < configuration_service_1.ConfigurationService.getConfiguration().tesla.minAmps) {
                newAmps = configuration_service_1.ConfigurationService.getConfiguration().tesla.minAmps;
            }
            if (newAmps > configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps) {
                newAmps = configuration_service_1.ConfigurationService.getConfiguration().tesla.maxAmps;
            }
            if (newAmps != TeslaService.last_receive_data.charge_state.charge_amps) {
                console.log("Tesla set amps to " + newAmps + '(instead of ' + TeslaService.last_receive_data.charge_state.charge_amps + ')');
                TeslaService.setChargeAmps(newAmps);
                TeslaService.lastChangeChargingAmps = new Date().getTime();
            }
        }
    }
    static isTeslaAtHome() {
        if (configuration_service_1.ConfigurationService.getConfiguration().tesla.useBleProxy) {
            return true;
        }
        const lat1 = TeslaService.last_receive_data.drive_state.latitude;
        const lon1 = TeslaService.last_receive_data.drive_state.longitude;
        const lat2 = configuration_service_1.ConfigurationService.getConfiguration().tesla.homeLatitude;
        const lon2 = configuration_service_1.ConfigurationService.getConfiguration().tesla.homeLongitude;
        if (lat2 === 0 && lon2 === 0) {
            return true;
        }
        let dist = Math.acos(Math.sin(lat1 * Math.PI / 180) * Math.sin(lat2 * Math.PI / 180) + Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) * Math.cos((lon2 - lon1) * Math.PI / 180)) * 6371;
        return dist < 0.5;
    }
    static addChargeValue(enphaseArray) {
        let currentIndex = 0;
        for (let i = 0; i < enphaseArray.length; i++) {
            enphaseArray[i].w_charge = 0;
        }
        for (let i = 1; i < enphaseArray.length; i++) {
            let timestamp = enphaseArray[i - 1].timestamp;
            while (currentIndex < TeslaService.chargeData.length && TeslaService.chargeData[currentIndex].timestamp < timestamp) {
                currentIndex++;
            }
            if (currentIndex >= TeslaService.chargeData.length) {
                return enphaseArray;
            }
            if (TeslaService.chargeData[currentIndex].timestamp < enphaseArray[i].timestamp) {
                enphaseArray[i].w_charge = TeslaService.chargeData[currentIndex].w_charge;
            }
        }
        return enphaseArray;
    }
    static setConfiguration(configuration, attribModified, from) {
        if (attribModified.startsWith('"tesla.')) {
            TeslaService.connect();
        }
    }
    static checkModeSolarPlus() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        return __awaiter(this, void 0, void 0, function* () {
            if (TeslaService.chargingMode !== 'solarStart') {
                //console.log ('not charge start in solarStart mode because not in solarStart mode');
                return;
            }
            // If last time to start charge was less than one hour
            if (TeslaService.lastSolarPlusStartChargeTime > (new Date().getTime() - 3600 * 1000)) {
                //console.log ('not charge start in solarStart mode because last start too near');
                return;
            }
            // Check if production is enough to start charge
            let overProduction = enphase_data_service_1.EnphaseDataService.getOverproduction(configuration_service_1.ConfigurationService.getConfiguration().tesla.adaptChargeFrequency);
            if (overProduction && overProduction >= configuration_service_1.ConfigurationService.getConfiguration().tesla.minAmps * 220 + configuration_service_1.ConfigurationService.getConfiguration().tesla.chargeOffset) {
                // Ok, overproduction is enough so try to start charge
                // If already in charge then do nothing
                if (((_b = (_a = TeslaService.last_receive_data) === null || _a === void 0 ? void 0 : _a.charge_state) === null || _b === void 0 ? void 0 : _b.charging_state) === "Starting" || ((_d = (_c = TeslaService.last_receive_data) === null || _c === void 0 ? void 0 : _c.charge_state) === null || _d === void 0 ? void 0 : _d.charging_state) === "Charging") {
                    //console.log ('not charge start in solarStart mode because already in charge mode');
                    return;
                }
                else {
                    if (((_f = (_e = this.last_receive_data) === null || _e === void 0 ? void 0 : _e.charge_state) === null || _f === void 0 ? void 0 : _f.charge_limit_soc) > ((_h = (_g = this.last_receive_data) === null || _g === void 0 ? void 0 : _g.charge_state) === null || _h === void 0 ? void 0 : _h.usable_battery_level)) {
                        // Start charge
                        console.log('charge start in solarStart mode');
                        let ret = yield TeslaService.startCharge();
                        if (ret) {
                            TeslaService.lastSolarPlusStartChargeTime = new Date().getTime();
                        }
                        // Try to get data in 30 seconds
                        TeslaService.mode = TeslaService.MODE_GET_DATA;
                        TeslaService.next_action_time = new Date().getTime() + 30000;
                    }
                    else {
                        // Vehicle battery full so do nothing
                        console.log('do not launch charge because limit is ' + ((_k = (_j = this.last_receive_data) === null || _j === void 0 ? void 0 : _j.charge_state) === null || _k === void 0 ? void 0 : _k.charge_limit_soc) + ' and current value is ' + ((_m = (_l = this.last_receive_data) === null || _l === void 0 ? void 0 : _l.charge_state) === null || _m === void 0 ? void 0 : _m.usable_battery_level));
                    }
                }
            }
            else {
                // not charge start in solarStart mode because overproduction to low
            }
        });
    }
}
exports.TeslaService = TeslaService;
/** Le token pour acceder à l'API */
TeslaService.access_token = '';
/** L'id du vehicule. */
TeslaService.vehicle_id = '';
/** Les dernière données recues. */
TeslaService.last_receive_data = {};
TeslaService.nbWakeUp = 0;
TeslaService.wasInCharge = false;
TeslaService.MODE_INIT = 0;
TeslaService.MODE_GET_DATA = 10;
TeslaService.MODE_GET_DATA_NOW = 11;
TeslaService.MODE_WAIT_ONLINE = 20;
TeslaService.MODE_GET_CONTINUS_DATA = 30;
TeslaService.MODE_REFRESH_TOKEN = 40;
TeslaService.mode = TeslaService.MODE_INIT;
TeslaService.previousMode = TeslaService.mode;
TeslaService.STEP_CONNECT = 0;
TeslaService.STEP_GET_VEHICLES = 10;
TeslaService.STEP_GET_ONLINE = 20;
TeslaService.STEP_WAKE_UP = 30;
TeslaService.STEP_GET_DATA = 40;
TeslaService.step = TeslaService.STEP_CONNECT;
TeslaService.previousStep = TeslaService.step;
TeslaService.next_action_time = 0;
TeslaService.last_success_get_data = 0;
TeslaService.max_time_between_get_data = 6 * 60 * 60 * 1000; // 6 heures
TeslaService.chargingMode = "normal";
TeslaService.lastChangeChargingAmps = 0;
TeslaService.lastSolarPlusStartChargeTime = 0;
TeslaService.chargeData = [];
/**
 * Ouvrir le trunk.
 */
TeslaService.trunkOpen = function () {
    let xhr = new XMLHttpRequest();
    xhr.onload = function () {
        TeslaService.log("reponse recu de tesla : " + this.responseText);
        if (this.status === 200) {
            let vehicleResponse = JSON.parse(this.responseText).response;
            if (vehicleResponse.result) {
                TeslaService.log("Coffre arière ouvert");
            }
        }
    };
    xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/actuate_trunk?which_trunk=\"rear\"");
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
    xhr.send("{\"which_trunk\":\"rear\"}");
};
/**
 * Deverrouiller les portes.
 */
TeslaService.doorUnlock = function () {
    let xhr = new XMLHttpRequest();
    xhr.onload = function () {
        TeslaService.log("reponse recu de tesla : " + this.responseText);
        if (this.status === 200) {
            let vehicleResponse = JSON.parse(this.responseText).response;
            if (vehicleResponse.result) {
                TeslaService.log("Portes déverrouillées");
            }
        }
    };
    xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/door_unlock");
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
    xhr.send();
};
/**
 * Ouvrir les fenêtre.
 */
TeslaService.ventOpen = function () {
    let xhr = new XMLHttpRequest();
    xhr.onload = function () {
        TeslaService.log("reponse recu de tesla : " + this.responseText);
        if (this.status === 200) {
            let vehicleResponse = JSON.parse(this.responseText).response;
            if (vehicleResponse.result) {
                TeslaService.log("Fenêtre ouvertes");
            }
        }
    };
    var obj = {};
    obj.command = "vent";
    obj.lat = 0;
    obj.long = 0;
    xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/window_control");
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
    xhr.send(JSON.stringify(obj));
};
/**
 * Stop le mode sentinel.
 */
TeslaService.stopSentinel = function () {
    var xhr = new XMLHttpRequest();
    xhr.onload = function () {
        TeslaService.log("reponse recu de tesla : " + this.responseText);
        if (this.status === 200) {
            let vehicleResponse = JSON.parse(this.responseText).response;
            if (vehicleResponse.result) {
                TeslaService.log("Sentinel arrêtée");
            }
        }
    };
    xhr.open("POST", "https://owner-api.teslamotors.com/api/1/vehicles/" + TeslaService.vehicle_id + "/command/set_sentry_mode");
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Authorization", "Bearer " + TeslaService.access_token);
    xhr.send("{\"on\":false}");
};
/**
 * Check if we need to get data.
 */
TeslaService.checkGetNewData = function () {
    let d = new Date();
    if (d.getTime() > TeslaService.next_action_time) {
        switch (TeslaService.step) {
            case TeslaService.STEP_CONNECT:
                TeslaService.connect();
                break;
            case TeslaService.STEP_GET_VEHICLES:
                TeslaService.getVehicleList();
                break;
            case TeslaService.STEP_GET_ONLINE:
                TeslaService.getOnline();
                break;
            case TeslaService.STEP_WAKE_UP:
                TeslaService.wakeUpVehicle();
                break;
            case TeslaService.STEP_GET_DATA:
                TeslaService.getVehicleData();
                break;
        }
    }
};
/**
 * Initialise the tesla object.
 */
TeslaService.init = function () {
    console.log('Init of TeslaService');
    TeslaService.chargingMode = configuration_service_1.ConfigurationService.getConfiguration().tesla.chargingMode;
    if (configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        setInterval(TeslaService.checkGetNewData, 1000);
    }
};
/**
 * Defini le temps entre chaque reveille forcé
 */
TeslaService.setForcedWakeUp = function (delayMs) {
    TeslaService.max_time_between_get_data = delayMs;
};
